@@ -26,9 +26,6 @@ from jax._src.numpy.lax_numpy import (
     NINF as NINF,
     NZERO as NZERO,
     PZERO as PZERO,
-    abs as abs,
-    absolute as absolute,
-    add as add,
     all as all,
     allclose as allclose,
     alltrue as alltrue,
@@ -40,13 +37,6 @@ from jax._src.numpy.lax_numpy import (
     apply_along_axis as apply_along_axis,
     apply_over_axes as apply_over_axes,
     arange as arange,
-    arccos as arccos,
-    arccosh as arccosh,
-    arcsin as arcsin,
-    arcsinh as arcsinh,
-    arctan as arctan,
-    arctan2 as arctan2,
-    arctanh as arctanh,
     argmax as argmax,
     argmin as argmin,
     argsort as argsort,
@@ -66,21 +56,14 @@ from jax._src.numpy.lax_numpy import (
     bartlett as bartlett,
     bfloat16 as bfloat16,
     bincount as bincount,
-    bitwise_and as bitwise_and,
-    bitwise_not as bitwise_not,
-    bitwise_or as bitwise_or,
-    bitwise_xor as bitwise_xor,
     blackman as blackman,
     block as block,
     bool_ as bool_,
     broadcast_arrays as broadcast_arrays,
     broadcast_shapes as broadcast_shapes,
     broadcast_to as broadcast_to,
-    c_ as c_,
     can_cast as can_cast,
-    cbrt as cbrt,
     cdouble as cdouble,
-    ceil as ceil,
     character as character,
     choose as choose,
     clip as clip,
@@ -91,15 +74,10 @@ from jax._src.numpy.lax_numpy import (
     complexfloating as complexfloating,
     compress as compress,
     concatenate as concatenate,
-    conj as conj,
-    conjugate as conjugate,
     convolve as convolve,
     copy as copy,
-    copysign as copysign,
     corrcoef as corrcoef,
     correlate as correlate,
-    cos as cos,
-    cosh as cosh,
     count_nonzero as count_nonzero,
     cov as cov,
     cross as cross,
@@ -107,8 +85,6 @@ from jax._src.numpy.lax_numpy import (
     cumprod as cumprod,
     cumproduct as cumproduct,
     cumsum as cumsum,
-    deg2rad as deg2rad,
-    degrees as degrees,
     delete as delete,
     diag as diag,
     diagflat as diagflat,
@@ -117,8 +93,6 @@ from jax._src.numpy.lax_numpy import (
     diagonal as diagonal,
     diff as diff,
     digitize as digitize,
-    divide as divide,
-    divmod as divmod,
     dot as dot,
     double as double,
     dsplit as dsplit,
@@ -130,15 +104,10 @@ from jax._src.numpy.lax_numpy import (
     einsum_path as einsum_path,
     empty as empty,
     empty_like as empty_like,
-    equal as equal,
     euler_gamma as euler_gamma,
-    exp as exp,
-    exp2 as exp2,
     expand_dims as expand_dims,
-    expm1 as expm1,
     extract as extract,
     eye as eye,
-    fabs as fabs,
     finfo as finfo,
     fix as fix,
     flatnonzero as flatnonzero,
@@ -150,14 +119,9 @@ from jax._src.numpy.lax_numpy import (
     float32 as float32,
     float64 as float64,
     float_ as float_,
-    float_power as float_power,
     floating as floating,
-    floor as floor,
-    floor_divide as floor_divide,
     fmax as fmax,
     fmin as fmin,
-    fmod as fmod,
-    frexp as frexp,
     full as full,
     full_like as full_like,
     gcd as gcd,
@@ -165,23 +129,17 @@ from jax._src.numpy.lax_numpy import (
     geomspace as geomspace,
     get_printoptions as get_printoptions,
     gradient as gradient,
-    greater as greater,
-    greater_equal as greater_equal,
     hamming as hamming,
     hanning as hanning,
-    heaviside as heaviside,
     histogram as histogram,
     histogram_bin_edges as histogram_bin_edges,
     histogram2d as histogram2d,
     histogramdd as histogramdd,
     hsplit as hsplit,
     hstack as hstack,
-    hypot as hypot,
     i0 as i0,
     identity as identity,
     iinfo as iinfo,
-    imag as imag,
-    index_exp as index_exp,
     indices as indices,
     inexact as inexact,
     in1d as in1d,
@@ -196,16 +154,10 @@ from jax._src.numpy.lax_numpy import (
     integer as integer,
     interp as interp,
     intersect1d as intersect1d,
-    invert as invert,
     isclose as isclose,
     iscomplex as iscomplex,
     iscomplexobj as iscomplexobj,
-    isfinite as isfinite,
     isin as isin,
-    isinf as isinf,
-    isnan as isnan,
-    isneginf as isneginf,
-    isposinf as isposinf,
     isreal as isreal,
     isrealobj as isrealobj,
     isscalar as isscalar,
@@ -216,39 +168,19 @@ from jax._src.numpy.lax_numpy import (
     kaiser as kaiser,
     kron as kron,
     lcm as lcm,
-    ldexp as ldexp,
-    left_shift as left_shift,
-    less as less,
-    less_equal as less_equal,
     lexsort as lexsort,
     linspace as linspace,
     load as load,
-    log as log,
-    log10 as log10,
-    log1p as log1p,
-    log2 as log2,
-    logaddexp as logaddexp,
-    logaddexp2 as logaddexp2,
-    logical_and as logical_and,
-    logical_not as logical_not,
-    logical_or as logical_or,
-    logical_xor as logical_xor,
     logspace as logspace,
     mask_indices as mask_indices,
     matmul as matmul,
     max as max,
-    maximum as maximum,
     mean as mean,
     median as median,
     meshgrid as meshgrid,
-    mgrid as mgrid,
     min as min,
-    minimum as minimum,
-    mod as mod,
-    modf as modf,
     moveaxis as moveaxis,
     msort as msort,
-    multiply as multiply,
     nan as nan,
     nan_to_num as nan_to_num,
     nanargmax as nanargmax,
@@ -267,14 +199,10 @@ from jax._src.numpy.lax_numpy import (
     nanvar as nanvar,
     ndarray as ndarray,
     ndim as ndim,
-    negative as negative,
     newaxis as newaxis,
-    nextafter as nextafter,
     nonzero as nonzero,
-    not_equal as not_equal,
     number as number,
     object_ as object_,
-    ogrid as ogrid,
     ones as ones,
     ones_like as ones_like,
     outer as outer,
@@ -283,36 +211,18 @@ from jax._src.numpy.lax_numpy import (
     percentile as percentile,
     pi as pi,
     piecewise as piecewise,
-    poly as poly,
-    polyadd as polyadd,
-    polyder as polyder,
-    polyfit as polyfit,
-    polyint as polyint,
-    polymul as polymul,
-    polysub as polysub,
-    polyval as polyval,
-    positive as positive,
-    power as power,
     printoptions as printoptions,
     prod as prod,
     product as product,
     promote_types as promote_types,
     ptp as ptp,
     quantile as quantile,
-    r_ as r_,
-    rad2deg as rad2deg,
-    radians as radians,
     ravel as ravel,
     ravel_multi_index as ravel_multi_index,
-    real as real,
-    reciprocal as reciprocal,
-    remainder as remainder,
     repeat as repeat,
     reshape as reshape,
     resize as resize,
     result_type as result_type,
-    right_shift as right_shift,
-    rint as rint,
     roll as roll,
     rollaxis as rollaxis,
     rot90 as rot90,
@@ -327,31 +237,20 @@ from jax._src.numpy.lax_numpy import (
     setdiff1d as setdiff1d,
     setxor1d as setxor1d,
     shape as shape,
-    sign as sign,
-    signbit as signbit,
-    s_ as s_,
     signedinteger as signedinteger,
-    sin as sin,
-    sinc as sinc,
     single as single,
-    sinh as sinh,
     size as size,
     sometrue as sometrue,
     sort as sort,
     sort_complex as sort_complex,
     split as split,
-    sqrt as sqrt,
-    square as square,
     squeeze as squeeze,
     stack as stack,
     std as std,
-    subtract as subtract,
     sum as sum,
     swapaxes as swapaxes,
     take as take,
     take_along_axis as take_along_axis,
-    tan as tan,
-    tanh as tanh,
     tensordot as tensordot,
     tile as tile,
     trace as trace,
@@ -365,7 +264,6 @@ from jax._src.numpy.lax_numpy import (
     triu as triu,
     triu_indices as triu_indices,
     triu_indices_from as triu_indices_from,
-    true_divide as true_divide,
     trunc as trunc,
     uint as uint,
     uint16 as uint16,
@@ -389,7 +287,118 @@ from jax._src.numpy.lax_numpy import (
     _NOT_IMPLEMENTED,
 )
 
-from jax._src.numpy.polynomial import roots as roots
+from jax._src.numpy.index_tricks import (
+  c_ as c_,
+  index_exp as index_exp,
+  mgrid as mgrid,
+  ogrid as ogrid,
+  r_ as r_,
+  s_ as s_,
+)
+
+from jax._src.numpy.polynomial import (
+    poly as poly,
+    polyadd as polyadd,
+    polyder as polyder,
+    polyfit as polyfit,
+    polyint as polyint,
+    polymul as polymul,
+    polysub as polysub,
+    polyval as polyval,
+    roots as roots,
+)
+
+from jax._src.numpy.ufuncs import (
+    abs as abs,
+    absolute as absolute,
+    add as add,
+    arccos as arccos,
+    arccosh as arccosh,
+    arcsin as arcsin,
+    arcsinh as arcsinh,
+    arctan as arctan,
+    arctan2 as arctan2,
+    arctanh as arctanh,
+    bitwise_and as bitwise_and,
+    bitwise_not as bitwise_not,
+    bitwise_or as bitwise_or,
+    bitwise_xor as bitwise_xor,
+    cbrt as cbrt,
+    ceil as ceil,
+    conj as conj,
+    conjugate as conjugate,
+    copysign as copysign,
+    cos as cos,
+    cosh as cosh,
+    deg2rad as deg2rad,
+    degrees as degrees,
+    divide as divide,
+    divmod as divmod,
+    equal as equal,
+    exp as exp,
+    exp2 as exp2,
+    expm1 as expm1,
+    fabs as fabs,
+    float_power as float_power,
+    floor as floor,
+    floor_divide as floor_divide,
+    fmod as fmod,
+    frexp as frexp,
+    greater as greater,
+    greater_equal as greater_equal,
+    heaviside as heaviside,
+    hypot as hypot,
+    imag as imag,
+    invert as invert,
+    isfinite as isfinite,
+    isinf as isinf,
+    isnan as isnan,
+    isneginf as isneginf,
+    isposinf as isposinf,
+    ldexp as ldexp,
+    left_shift as left_shift,
+    less as less,
+    less_equal as less_equal,
+    log as log,
+    log10 as log10,
+    log1p as log1p,
+    log2 as log2,
+    logaddexp as logaddexp,
+    logaddexp2 as logaddexp2,
+    logical_and as logical_and,
+    logical_not as logical_not,
+    logical_or as logical_or,
+    logical_xor as logical_xor,
+    maximum as maximum,
+    minimum as minimum,
+    mod as mod,
+    modf as modf,
+    multiply as multiply,
+    negative as negative,
+    nextafter as nextafter,
+    not_equal as not_equal,
+    positive as positive,
+    power as power,
+    rad2deg as rad2deg,
+    radians as radians,
+    real as real,
+    reciprocal as reciprocal,
+    remainder as remainder,
+    right_shift as right_shift,
+    rint as rint,
+    sign as sign,
+    signbit as signbit,
+    sin as sin,
+    sinc as sinc,
+    sinh as sinh,
+    sqrt as sqrt,
+    square as square,
+    subtract as subtract,
+    tan as tan,
+    tanh as tanh,
+    true_divide as true_divide,
+)
+
 from jax._src.numpy.vectorize import vectorize as vectorize
 
 # TODO(phawkins): remove this import after fixing users.
